/* spmfilter - mail filtering framework
 * Copyright (C) 2009-2010 Werner Detter and SpaceNet AG
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <string.h>
#include <stdlib.h>
#include <glib.h>
#include <spmfilter.h>
#define THIS_MODULE "smtpbcc"

/* start here */
int load(SMFSession_T *session) {
	int i;
	int ret;
	char *auth_user = NULL;
	char *auth_pass = NULL;
	char *smtp_host = NULL;
	SMFMessageEnvelope_T *msg = smf_message_envelope_new();
	/* logic check if all needed parameters are found in config */
	if (smf_settings_group_load(THIS_MODULE) !=0 ) {
		TRACE(TRACE_ERR, "no configuration found in spmfilter.conf");
		return -1;
	}
	/* check if smtp_host is defined in config and initialize, quit if missing */
	if(smf_settings_group_get_string("smtp_host") != NULL) {
		smtp_host = smf_settings_group_get_string("smtp_host");
		TRACE(TRACE_DEBUG, "smtp_host: %s", smtp_host);
	} else {
		TRACE(TRACE_ERR, "error: \"smtp_host\" not found in config!");
		return -1;
	}
	if(smf_settings_group_get_string("auth_user") != NULL &&
	   smf_settings_group_get_string("auth_pass") == NULL) {
		TRACE(TRACE_ERR, "error: \"auth_user\" defined in "
		"config but \"auth_pass\" is missing!");
		return -1;
	} else if(smf_settings_group_get_string("auth_user") == NULL &&
		smf_settings_group_get_string("auth_pass") != NULL) {
		TRACE(TRACE_ERR, "error: \"auth_pass\" defined in "
		"config but \"auth_user\" is missing!");
		return -1;
	} else {
		/* initialize optional parameters */
		auth_user = smf_settings_group_get_string("auth_user");
		auth_pass = smf_settings_group_get_string("auth_pass");
		TRACE(TRACE_DEBUG, "auth_user: %s", auth_user);
		TRACE(TRACE_DEBUG, "auth_pass: %s", auth_pass);
	}
	if (session->envelope_from != NULL) {
		msg->from = g_strdup(session->envelope_from->addr);
	} else if (session->message_from != NULL) {
		msg->from = g_strdup(session->message_from->addr);
	}
	msg->nexthop = g_strdup(smtp_host);
	if(auth_user != NULL && auth_pass != NULL ) {
		msg->auth_user = g_strdup(auth_user);
		msg->auth_pass = g_strdup(auth_pass);
	}
	if (session->envelope_from != NULL) {
		for(i=0; i < session->envelope_to_num; i++) {
			smf_message_envelope_add_rcpt(msg, session->envelope_to[i]->addr);
			TRACE(TRACE_DEBUG, "added recipient %d: %s", i,
			session->envelope_to[i]->addr);
		}
	} else if (session->message_from != NULL) {
		for(i=0; i < session->message_to_num; i++) {
			smf_message_envelope_add_rcpt(msg, session->message_to[i]->addr);
			TRACE(TRACE_DEBUG, "added recipient %d: %s", i,
			session->message_to[i]->addr);
		}
	}
	
	/* send copy of the message to smtp_host */
	msg->message = smf_session_get_message();
	ret = smf_message_deliver(msg);
	/* clean up */
	smf_message_envelope_unref(msg);
	
	if (ret != 0) {
		TRACE(TRACE_ERR,"delivery to %s failed!", smtp_host);
		ret = -1;
	} else {
		TRACE(TRACE_INFO, "successfully delivered message to %s", smtp_host);
		ret = 0;
	}
	return ret;
}